@@ -151,8 +151,8 @@ class SFREAD:
             return
 
         logger.info('Fetching SF %s', self.sfile)
-        self.objects = [ lbl for lbl in self.sfh['obj_nam'] if self.sfh[lbl].obj_type in [6, 7, 8, 13] ]
-        self.parsets = [ lbl for lbl in self.sfh['obj_nam'] if self.sfh[lbl].obj_type in [3, 4] ]
+        self.objects = [ lbl for lbl, sfo in self.sfh.items() if sfo.obj_type in [6, 7, 8, 13] ]
+        self.parsets = [ lbl for lbl, sfo in self.sfh.items() if sfo.obj_type in [3, 4] ]
         if 'ADDRLEN' in self.sfh.keys():
             size_fac = sfdics.addrsizes[self.sfh['ADDRLEN'].addrlen]
             for obj in self.sfh.keys():
@@ -201,8 +201,7 @@ class SFREAD:
         if otyp == 8:
             return self.getobject(obj)
         elif otyp in (6, 7, 13):
-            rels = self.getrelnames(obj)
-            for rel in rels:
+            for rel in sfo.relations:
                 if self.sfh[rel].obj_type == 8:
                     return self.getobject(rel)
         return None
@@ -222,8 +221,7 @@ class SFREAD:
         if otyp == 13:
             return self.getobject(obj)
         elif otyp in (6, 7):
-            rels = self.getrelnames(obj)
-            for rel in rels:
+            for rel in sfo.relations:
                 if self.sfh[rel].obj_type == 13:
                     return self.getobject(rel)
         return None
@@ -314,7 +312,7 @@ class SFREAD:
         Returns the time-array in [s] for TB of type PPG_prog
         """
 
-        for rel in self.getrelnames(tbobj):
+        for rel in self.sfh[tbobj].relations:
             if self.sfh[rel].obj_type == 3:
                 ppg = self.getdevice(rel)
                 npt = self.sfh[tbobj].n_pre
@@ -386,7 +384,7 @@ class SFREAD:
         Returns coefficients for signal(group) calibration
         """
         obj = str_byt.to_str(obj)
-        for rel in self.getrelnames(obj):
+        for rel in self.sfh[obj].relations:
             if self.sfh[rel].obj_type == 4:
                 caltyp = str_byt.to_str(sfdics.cal_type[self.sfh[rel].cal_type])
                 if caltyp == 'LinCalib':
@@ -437,11 +435,10 @@ class SFREAD:
                 sfmt = sfdics.fmt2struc[dfmt]
                 val_len = n_items + 2
                 bytlen = val_len * np.dtype(sfmt).itemsize
-                fmt = '>%d%s' %(val_len, sfmt)
-                if dfmt == 7:
+                if dfmt == 7: # logical, bug if n_items > 1?
                     data = unpack(sfmt, buf[j0+5: j0+6])[0]
                 else:
-                    data = unpack(fmt, buf[j0: j0+bytlen])
+                    data = np.ndarray((val_len, ), '>%s' %sfmt, buf[j0: j0+bytlen], order='F')
                     data = np.squeeze(data[2:])
                 dj0 = 8 * ( (bytlen + 7)//8 )
                 j0 += dj0
@@ -476,7 +473,7 @@ class SFREAD:
 
     def getlist_by_type(self, obj_type=7):
 
-        return [lbl for lbl in self.sfh['obj_nam'] if self.sfh[lbl].obj_type == obj_type]
+        return [lbl for lbl, sfo in self.sfh.items() if sfo.obj_type == obj_type]
 
 
     def getobjname(self, jobj):
@@ -484,26 +481,9 @@ class SFREAD:
         Returns the object name for an inpur object ID
         """
 
-        return self.sfh['obj_nam'][jobj]
-
-
-    def getobjid(self, lbl):
-        """
-        Returns the object ID for a given objeect
-        """
-
-        lbl = str_byt.to_str(lbl)
-        return self.sfh['obj_id'][lbl]
-
-
-    def getrelnames(self, lbl):
-        """
-        Returns the relation names of a given object
-        """
-
-        lbl = str_byt.to_str(lbl)
-        rels_id = self.sfh[lbl].rel
-        return [self.getobjname(jobj) for jobj in rels_id if jobj < 65535]
+        for lbl, sfo in self.sfh.items():
+            if sf.objid == jobj:
+                return lbl
 
 
     def getdevice(self, lbl):
@@ -536,7 +516,7 @@ class SFREAD:
         if self.sfh[obj].obj_type == 8:
             tb = obj
         else:
-            for rel_obj in self.getrelnames(obj):
+            for rel_obj in self.sfh[obj].relations:
                 if self.sfh[rel_obj].obj_type == 8: # related TB
                     tb = rel_obj
                     break
@@ -545,7 +525,7 @@ class SFREAD:
         else: # try a direct relation
             obj2 = obj
 
-        for rel_obj in self.getrelnames(obj2):
+        for rel_obj in self.sfh[obj2].relations:
             if self.sfh[rel_obj].obj_type == 3: # related device
                 ps = self.getparset(rel_obj)
                 if 'TS06' in ps.keys():
@@ -574,8 +554,7 @@ class SFREAD:
         if otyp != 6:
             return False # Objects other than sig, sgr, tbase
 
-        rels = self.getrelnames(obj)
-        for rel in rels:
+        for rel in sfo.relations:
             if self.sfh[rel].obj_type == 8:
                 shape_arr = np.array(sfo.index[::-1][:sfo.num_dims])
                 nt = self.sfh[rel].n_steps
@@ -599,8 +578,7 @@ class SFREAD:
         if otyp != 6:
             return False # Objects other than sig, sgr, tbase
 
-        rels = self.getrelnames(obj)
-        for rel in rels:
+        for rel in sfo.relations:
             if self.sfh[rel].obj_type == 8:
                 shape_arr = np.array(sfo.index[::-1][:sfo.num_dims])
                 nt = self.sfh[rel].n_steps
